﻿using System;
using System.Windows;
using Microsoft.Practices.Composite.UnityExtensions;
using Microsoft.Practices.EnterpriseLibrary.ExceptionHandling;

namespace BlueMarble.Genesis.Shell
{
    /// <summary>
    /// Interaction logic for App.xaml
    /// </summary>
    public partial class App : Application
    {
        protected override void OnStartup(StartupEventArgs e)
        {
            base.OnStartup(e);

#if (DEBUG)
            RunInDebugMode();
#else
            RunInReleaseMode();
#endif

            this.ShutdownMode = ShutdownMode.OnMainWindowClose;
        }

        private static void RunInDebugMode()
        {
                UnityBootstrapper bootstrapper = new ShellBootstrapper();
                bootstrapper.Run();
        }

        private static void RunInReleaseMode()
        {
            AppDomain.CurrentDomain.UnhandledException += AppDomainUnhandledException;
            try
            {
                UnityBootstrapper bootstrapper = new ShellBootstrapper();
                bootstrapper.Run();
            }
            catch (Exception ex)
            {
                HandleException(ex);
            }
        }

        private static void AppDomainUnhandledException(object sender, UnhandledExceptionEventArgs e)
        {
            HandleException(e.ExceptionObject as Exception);
        }

        private static void HandleException(Exception ex)
        {
            if (ex == null)
                return;

            ExceptionPolicy.HandleException(ex, "Default Policy");
            MessageBox.Show(BlueMarble.Genesis.Shell.Resources.ErrorStrings.UnhandledException);
            Environment.Exit(1);
        }
    }
}
