#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Genesis.Common;
using Genesis.Common.Interfaces;
using Genesis.Common.Classes;

namespace Genesis.Client.Common.Classes
{
    public partial class MdiChildBase : System.Windows.Forms.Form, IMdiChild
    {
        protected string pageTitle;
        protected bool canBrowseBack;
        protected bool canBrowseForward;
        protected bool canRefresh;
        protected bool canPrint;
        protected bool canSave;
        protected bool isBusy;
        protected List<PageProperty> pageProperties;
        protected IHost host;
        protected IPage page;
        protected Image image;

        protected string galleryGroupName;
        protected string galleryGroupText;

        public MdiChildBase()
        {
            InitializeComponent();

            pageProperties = new List<PageProperty>();

            pageTitle = "Page title not set";
            canBrowseBack = false;
            canBrowseForward = false;
            canRefresh = false;
            canPrint = false;
            canSave = false;
            isBusy = false;

            string imageFileName = this.GetImageFileName();
            if (imageFileName.Length > 0)
            {
                image = Image.FromFile(imageFileName);
            }

            galleryGroupName = "Default";
            galleryGroupText = "<b>Default Group</b>";
        }

        #region IMdiChild Members

        public virtual bool CanBrowseBack()
        {
            return canBrowseBack;
        }

        public virtual bool CanBrowseForward()
        {
            return canBrowseForward;
        }

        public virtual void BrowseBack()
        {
        }

        public virtual void BrowseForward()
        {
        }

        public virtual bool CanRefresh()
        {
            return canRefresh;
        }

        public virtual bool CanPrint()
        {
            return canRefresh;
        }

        public virtual void Print()
        {
        }

        public virtual void Refresh()
        {
        }

        public virtual bool IsBusy()
        {
            return isBusy;
        }

        public virtual bool CanSave()
        {
            return canSave;
        }

        public virtual void Save()
        {
        }

        public virtual void Reload()
        {
        }

        public virtual void Closing()
        {
        }

        public string GetGalleryGroupName()
        {
            return galleryGroupName;
        }

        public string GetGalleryGroupText()
        {
            return galleryGroupText;
        }

        public string GetPageTitle()
        {
            return pageTitle;
        }

        public void ResetPageProperties()
        {
            pageProperties.Clear();
        }

        public bool ThumbnailCallback()
        {
            return false;
        }

        public string GetImageFileName()
        {
            string fileName = "";

            if (this.getPageProperty("WindowImageFileName") != null)
            {
                fileName = this.getPageProperty("ImageFileName").Value.ToString();
            }

            string returnValue = AppDomain.CurrentDomain.BaseDirectory + fileName;

            if (fileName.Length == 0)
            {
                returnValue = "";
            }

            return returnValue;
        }

        public Image GetWindowListImageThumbnail(Image Image, int Width, int Height)
        {
            Image.GetThumbnailImageAbort myCallback = new Image.GetThumbnailImageAbort(ThumbnailCallback);

            return Image.GetThumbnailImage(Width, Height, myCallback, IntPtr.Zero);
        }

        public Image GetWindowListImage(int Width, int Height)
        {
            if (this.MdiParent.ActiveMdiChild == this)
            {
                Genesis.Client.Common.Classes.ScreenCapture screenCapture = new ScreenCapture();

                image = screenCapture.CaptureWindow(this.Handle);

                return GetWindowListImageThumbnail(image, Width, Height);
            }
            else
            {
                return GetWindowListImageThumbnail(image, Width, Height);
            }
        }

        public List<PageProperty> PageProperties
        {
            get
            {
                return pageProperties;
            }
            set
            {
                pageProperties = value;
            }
        }

        public Genesis.Common.Interfaces.IHost Host
        {
            get
            {
                return host;
            }
            set
            {
                host = value;
            }
        }

        public Genesis.Common.Interfaces.IPage Page
        {
            get
            {
                return page;
            }
            set
            {
                page = value;

                this.pageTitle = page.GetPageTitle();
                this.canBrowseBack = page.CanBrowseBack();
                this.canBrowseForward = page.CanBrowseForward();
                this.canSave = page.CanSave();

                this.Text = page.GetPageTitle();

                ((System.Windows.Forms.UserControl)page).Dock = DockStyle.Fill;

                this.Controls.Clear();

                this.Controls.Add((System.Windows.Forms.UserControl)page);
            }
        }

        public string GalleryGroupName
        {
            get { return this.galleryGroupName; }
            set { this.galleryGroupName = value; }
        }

        public string GalleryGroupText
        {
            get { return this.galleryGroupText; }
            set { this.galleryGroupText = value; }
        }

        #endregion

        protected PageProperty getPageProperty(string PropertyName)
        {
            for (int i = 0; i < pageProperties.Count; i++)
            {
                if (pageProperties[i].Name == PropertyName)
                {
                    return pageProperties[i];
                }
            }

            return null;
        }
    }
}