#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using Genesis.Common.Interfaces;
using Genesis.Common.Classes;

namespace Genesis.Client.Common.Classes
{
    public class PageBase: System.Windows.Forms.UserControl, IPage
    {
        protected string pageTitle;
        protected bool canBrowseBack;
        protected bool canBrowseForward;
        protected bool canSave;
        protected bool isBusy;
        protected List<PageProperty> pageProperties;
        protected IHost host;

        public PageBase()
        {
            this.InitializeComponent();

            pageProperties = new List<PageProperty>();

            pageTitle = "Page title not set";
            canBrowseBack = false;
            canBrowseForward = false;
            canSave = false;
            isBusy = false;
        }

        #region IPage Members

        public virtual bool CanBrowseBack()
        {
            return canBrowseBack;
        }

        public virtual bool CanBrowseForward()
        {
            return canBrowseForward;
        }

        public virtual void BrowseBack()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public virtual void BrowseForward()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public virtual bool CanRefresh()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public virtual void Refresh()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public virtual bool IsBusy()
        {
            return isBusy;
        }

        public virtual bool CanSave()
        {
            return canSave;
        }

        public virtual void Save()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        public virtual void Reload()
        {
        }

        public virtual void Closing()
        {
        }

        public string GetPageTitle()
        {
            return pageTitle;
        }

        public void ResetPageProperties()
        {
            pageProperties.Clear();
        }

        public List<PageProperty> PageProperties
        {
            get
            {
                return pageProperties;
            }
            set
            {
                pageProperties = value;
            }
        }

        public Genesis.Common.Interfaces.IHost Host
        {
            get
            {
                return host;
            }
            set
            {
                host = value;
            }
        }

        #endregion

        protected PageProperty getPageProperty(string PropertyName)
        {
            for (int i = 0; i < pageProperties.Count; i++)
            {
                if (pageProperties[i].Name == PropertyName)
                {
                    return pageProperties[i];
                }
            }

            return null;
        }

        private void InitializeComponent()
        {
            this.SuspendLayout();
            // 
            // PageBase
            // 
            this.AutoScroll = true;
            this.BackColor = System.Drawing.Color.Transparent;
            this.BackgroundImageLayout = System.Windows.Forms.ImageLayout.None;
            this.Name = "PageBase";
            this.Size = new System.Drawing.Size(600, 400);
            this.Load += new System.EventHandler(this.PageBase_Load);
            this.ResumeLayout(false);

        }

        private void PageBase_Load(object sender, EventArgs e)
        {
        }
    }
}
