#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace Genesis.Client.Common.Pages
{
    public partial class WebBrowser : Genesis.Client.Common.Classes.PageBase, Genesis.Common.Interfaces.IPage
    {
        public WebBrowser()
        {
            InitializeComponent();

            base.pageTitle = "Web Browser";
        }

        #region IPage Members

        public override bool CanBrowseBack()
        {
            return browser.CanGoBack;
        }

        public override bool CanBrowseForward()
        {
            return browser.CanGoForward;
        }

        public override bool CanRefresh()
        {
            return true;
        }

        public override void Refresh()
        {
            this.browser.Refresh();
        }

        public override void BrowseBack()
        {
            browser.GoBack();

            host.UpdateNavigation();
        }

        public override void BrowseForward()
        {
            browser.GoForward();

            host.UpdateNavigation();
        }

        void Genesis.Common.Interfaces.IPage.Save()
        {
            throw new Exception("The method or operation is not implemented.");
        }

        void Genesis.Common.Interfaces.IPage.Closing()
        {
            browser.Dispose();
        }

        #endregion

        public void Navigate()
        {
            string url = "";
            string urlParam = "?";
            string title = "";

            foreach (Genesis.Common.Classes.PageProperty pageProperty in this.pageProperties)
            {
                if (pageProperty.Name == "Url")
                {
                    url = (string)pageProperty.Value;
                }
                else if (pageProperty.Name == "PageTitle")
                {
                    title = (string)pageProperty.Value;
                }
                else
                {
                    urlParam += pageProperty.Name + "=" + System.Web.HttpUtility.UrlEncode((string)pageProperty.Value) + "&";
                }
            }

            if (title != null)
            {
                this.pageTitle = title;
            }

            if (url != null)
            {
                browser.Navigate(url + urlParam);
            }
        }

        private void webBrowser_DocumentCompleted(object sender, WebBrowserDocumentCompletedEventArgs e)
        {
            this.isBusy = false;

            this.pageTitle = browser.DocumentTitle;
            this.host.UpdateTitle(this);

            host.UpdateNavigation();
        }

        private void browser_Navigating(object sender, WebBrowserNavigatingEventArgs e)
        {
            this.isBusy = true;

            host.UpdateNavigation();

            if (e.Url.Scheme.ToLower() == "genesis")
            {
                e.Cancel = true;

                string command = e.Url.AbsoluteUri;

                this.host.ExecuteCommandUrl(command.Replace("/?", "?"));
            }
        }
    }
}
