#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;

namespace Genesis.Common.Classes
{
    public class GlobalDataSetFunctions: Interfaces.IDataContainer
    {
        protected DataSet globalDataSet;

        public GlobalDataSetFunctions(DataSet GlobalDataSet)
        {
            globalDataSet = GlobalDataSet;

            if (this.globalDataSet.Tables.IndexOf("RunningValues") == -1)
            {
                DataTable runningValuesTable = new DataTable("RunningValues");

                DataColumn parameterNameColumn = new DataColumn("Name", typeof(string));
                DataColumn parameterValueColumn = new DataColumn("Value", typeof(string));

                runningValuesTable.Columns.Add(parameterNameColumn);
                runningValuesTable.Columns.Add(parameterValueColumn);

                globalDataSet.Tables.Add(runningValuesTable);
            }
        }

        #region IDataContainer Members

        public System.Data.DataSet GlobalDataSet
        {
            get
            {
                return globalDataSet;
            }
            set
            {
                globalDataSet = value;
            }
        }

        public void SaveGlobalDataSet()
        {
            globalDataSet.WriteXml("datastore.xml");
        }

        public void LoadGlobalDataSet()
        {
            globalDataSet.ReadXml("datastore.xml");
        }

        #endregion

        public Guid GetActiveApplicationGuid()
        {
            if ((globalDataSet.Tables["Active_Application"] != null) && (globalDataSet.Tables["Active_Application"].Rows.Count == 1))
            {
                return (Guid)globalDataSet.Tables["Active_Application"].Rows[0]["ApplicationGuid"];
            }
            else
            {
                return Guid.Empty;
            }
        }

        public string GetActiveApplicationName()
        {
            if ((globalDataSet.Tables["Active_Application"] != null) && (globalDataSet.Tables["Active_Application"].Rows.Count == 1))
            {
                return (string)globalDataSet.Tables["Active_Application"].Rows[0]["ApplicationName"];
            }
            else
            {
                return "Genesis Smart Client";
            }
        }

        public Guid GetActiveSessionGuid()
        {
            if ((globalDataSet.Tables["Active_Application"] != null) && (globalDataSet.Tables["Active_Application"].Rows.Count == 1))
            {
                return (Guid)globalDataSet.Tables["Active_Application"].Rows[0]["SessionGuid"];
            }
            else
            {
                return Guid.Empty;
            }
        }

        public string GetActiveUserName()
        {
            if ((globalDataSet.Tables["Active_Application"] != null) && (globalDataSet.Tables["Active_Application"].Rows.Count == 1))
            {
                return (string)globalDataSet.Tables["Active_Application"].Rows[0]["UserName"];
            }
            else
            {
                return "Not logged in";
            }
        }

        public void MergeTables(string TableName, DataTable NewTable)
        {
            if (globalDataSet.Tables[TableName] == null)
            {
                globalDataSet.Tables.Add(NewTable);
            }
            else
            {
                DataTable oldTable = globalDataSet.Tables[TableName];

                for (int i = 0; i < NewTable.Rows.Count; i++)
                {
                    DataRow dr = NewTable.Rows[i];

                    DataRow newRow = oldTable.NewRow();

                    for (int j = 0; j < dr.ItemArray.Length; j++)
                    {
                        newRow[j] = dr[j];
                    }

                    oldTable.Rows.Add(newRow);
                }
            }
        }

        public string GetProperty(string PropertyName)
        {
            string resultValue = null;

            if (globalDataSet.Tables.IndexOf("RunningValues") > -1)
            {
                foreach (DataRow dr in globalDataSet.Tables["RunningValues"].Rows)
                {
                    if ((string)dr["Name"] == PropertyName)
                    {
                        if (dr["Value"] != DBNull.Value)
                        {
                            resultValue = (string)dr["Value"];
                        }
                        else
                        {
                            resultValue = null;
                        }
                    }
                }
            }

            return resultValue;
        }

        public void SetProperty(string PropertyName, string PropertyValue)
        {
            if (globalDataSet.Tables.IndexOf("RunningValues") > -1)
            {
                bool propertyFound = false;
                int propertyIndex = -1;

                for (int i = 0; i < globalDataSet.Tables["RunningValues"].Rows.Count; i++)
                {
                    DataRow dr = globalDataSet.Tables["RunningValues"].Rows[i];

                    if ((string)dr["Name"] == PropertyName)
                    {
                        dr["Value"] = PropertyValue;

                        propertyFound = true;
                        propertyIndex = i;
                    }
                }

                if (propertyFound)
                {
                    globalDataSet.Tables["RunningValues"].Rows[propertyIndex].Delete();
                    globalDataSet.Tables["RunningValues"].AcceptChanges();

                    propertyFound = false;
                }

                if (!propertyFound)
                {
                    DataRow newRow = globalDataSet.Tables["RunningValues"].NewRow();

                    newRow["Name"] = PropertyName;
                    newRow["Value"] = PropertyValue;

                    globalDataSet.Tables["RunningValues"].Rows.Add(newRow);
                }
            }
        }

        public void ClearProperty(string PropertyName)
        {
            if (globalDataSet.Tables.IndexOf("RunningValues") > -1)
            {
                bool propertyFound = false;
                int propertyIndex = -1;

                for (int i = 0; i < globalDataSet.Tables["RunningValues"].Rows.Count; i++)
                {
                    DataRow dr = globalDataSet.Tables["RunningValues"].Rows[i];

                    if ((string)dr["Name"] == PropertyName)
                    {
                        propertyFound = true;
                        propertyIndex = i;
                    }
                }

                if (propertyFound)
                {
                    globalDataSet.Tables["RunningValues"].Rows[propertyIndex].Delete();
                    globalDataSet.Tables["RunningValues"].AcceptChanges();
                }
            }
        }
    }
}
