#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using System.Configuration;

namespace Genesis.Common.Classes
{
    public class ServiceProviderBase: Interfaces.IServiceProvider
    {
        protected string serverAddress;
        protected System.Web.Services.Protocols.SoapHttpClientProtocol webService;
        protected Guid sessionGuid;
        protected Guid transactionGuid;

        protected string proxyServer;
        protected int proxyServerAddress;
        protected string proxyUsername;
        protected string proxyPassword;
        protected string proxyDomain;
        protected bool proxyShown;

        public ServiceProviderBase()
        {
            if (ConfigurationManager.AppSettings["GenesisServerAddress"] != null)
            {
                serverAddress = ConfigurationManager.AppSettings["GenesisServerAddress"];
            }
            else
            {
                serverAddress = "http://genesis.webservice.hilts.co.za/";
            }

            proxyServer = string.Empty;
            proxyServerAddress = 0;
            proxyUsername = string.Empty;
            proxyPassword = string.Empty;
            proxyDomain = string.Empty;
            proxyShown = false;

            if (System.IO.File.Exists(AppDomain.CurrentDomain.BaseDirectory + "\\Settings.xml"))
            {
                System.Xml.XmlDocument settingsDocument = new System.Xml.XmlDocument();

                settingsDocument.Load(AppDomain.CurrentDomain.BaseDirectory + "\\Settings.xml");

                System.Security.Cryptography.TripleDESCryptoServiceProvider tripleDESCryptoServiceProvider = new System.Security.Cryptography.TripleDESCryptoServiceProvider();

                tripleDESCryptoServiceProvider.Key = System.Text.UTF8Encoding.GetEncoding("utf-8").GetBytes("There1nceWa5@ManW1thout@");

                Classes.TrippleDESDocumentEncryption tripleDESDocumentEncryption = new Classes.TrippleDESDocumentEncryption(settingsDocument, tripleDESCryptoServiceProvider);

                tripleDESDocumentEncryption.Decrypt();

                if (settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy") != null)
                {
                    if (settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Server") != null)
                    {
                        proxyServer = settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Server/Address").InnerText;
                        proxyServerAddress = int.Parse(settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Server/Port").InnerText);
                    }

                    proxyUsername = settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Username").InnerText;
                    proxyPassword = settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Password").InnerText;

                    if (settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Domain") != null)
                    {
                        proxyDomain = settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Domain").InnerText;
                    }
                }
            } 
        }

        public ServiceProviderBase(string ServerAddress): this()
        {
            serverAddress = ServerAddress;
        }

        public void setWebServiceProxy()
        {
            if (proxyServer != string.Empty)
            {
                this.webService.Proxy = new System.Net.WebProxy(proxyServer);

                if (proxyServerAddress != 0)
                {
                    this.webService.Proxy = new System.Net.WebProxy(proxyServer, proxyServerAddress);
                }

                if (proxyUsername != string.Empty)
                {
                    this.webService.Proxy.Credentials = new System.Net.NetworkCredential(proxyUsername, proxyPassword);

                    if (proxyDomain != string.Empty)
                    {
                        this.webService.Proxy.Credentials = new System.Net.NetworkCredential(proxyUsername, proxyPassword, proxyDomain);
                    }
                }
            }
        }

        public Guid SessionGuid
        {
            get { return sessionGuid; }
            set { sessionGuid = value; }
        }

        public Guid TransactionGuid
        {
            get
            {
                return transactionGuid;
            }

            set
            {
                transactionGuid = value;
            }
        }

        #region IServiceProvider Members

        public string ServerAddress
        {
            get
            {
                return serverAddress;
            }
            set
            {
                serverAddress = value;
            }
        }

        public System.Web.Services.Protocols.SoapHttpClientProtocol WebService
        {
            get
            {
                return webService;
            }
            set
            {
                webService = value;
            }
        }

        #endregion
    }
}
