#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Genesis.Loader
{
    public partial class MainForm : Form
    {
        protected List<System.Diagnostics.Process> processList;
        protected System.Xml.XmlDocument sessionDocument;

        protected string[] loadingStrings;
        protected int currentLoadingIndex;
        protected DateTime lastCycleUpdate;

        static int cycleStyle;

        protected string proxyServer;
        protected int proxyServerAddress;
        protected string proxyUsername;
        protected string proxyPassword;
        protected string proxyDomain;
        protected bool proxyShown;

        public MainForm()
        {
            InitializeComponent();

            cycleStyle = 1;

            currentLoadingIndex = 0;

            loadingStrings = new string[] { "--", "\\", "|", "/" };

            lastCycleUpdate = DateTime.MinValue;

            sessionDocument = new System.Xml.XmlDocument();

            if (System.IO.File.Exists(AppDomain.CurrentDomain.BaseDirectory + "\\session.xml"))
            {
                sessionDocument.Load(AppDomain.CurrentDomain.BaseDirectory + "\\session.xml");
            }
            else
            {
                sessionDocument.LoadXml("<?xml version=\"1.0\" encoding=\"utf-8\" ?><root></root>");
            }

            outputListBox.Visible = false;
            this.Height = titlePanel.Height + 2;

            this.Opacity = 0;
            Application.DoEvents();
            this.Update();

            this.RepositionForm();

            processList = new List<System.Diagnostics.Process>();

            proxyServer = string.Empty;
            proxyServerAddress = 0;
            proxyUsername = string.Empty;
            proxyPassword = string.Empty;
            proxyDomain = string.Empty;
            proxyShown = false;

            if (System.IO.File.Exists(AppDomain.CurrentDomain.BaseDirectory + "\\Settings.xml"))
            {
                System.Xml.XmlDocument settingsDocument = new System.Xml.XmlDocument();

                settingsDocument.Load(AppDomain.CurrentDomain.BaseDirectory + "\\Settings.xml");

                System.Security.Cryptography.TripleDESCryptoServiceProvider tripleDESCryptoServiceProvider = new System.Security.Cryptography.TripleDESCryptoServiceProvider();

                tripleDESCryptoServiceProvider.Key = System.Text.UTF8Encoding.GetEncoding("utf-8").GetBytes("There1nceWa5@ManW1thout@");

                TrippleDESDocumentEncryption tripleDESDocumentEncryption = new TrippleDESDocumentEncryption(settingsDocument, tripleDESCryptoServiceProvider);

                tripleDESDocumentEncryption.Decrypt();

                if (settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy") != null)
                {
                    proxyServer = settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Server/Address").InnerText;
                    proxyServerAddress = int.Parse(settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Server/Port").InnerText);
                    proxyUsername = settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Username").InnerText;
                    proxyPassword = settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Password").InnerText;
                    proxyDomain = settingsDocument.DocumentElement.SelectSingleNode("/Settings/Proxy/Domain").InnerText;
                }
            }
        }

        public void ShowLoadingLabel()
        {
            loadingLabel.Visible = true;
        }

        public void HideLoadingLabel()
        {
            loadingLabel.Visible = false;
        }

        public void CycleLoadingLabel(int CycleStyle)
        {
            DateTime now = DateTime.Now;

            if ((new TimeSpan(now.Ticks - lastCycleUpdate.Ticks)).TotalMilliseconds > 500)
            {
                lastCycleUpdate = now;

                if (CycleStyle == 1)
                {
                    loadingLabel.Text = loadingStrings[currentLoadingIndex];

                    currentLoadingIndex++;

                    if (currentLoadingIndex >= loadingStrings.Length)
                    {
                        currentLoadingIndex = 0;
                    }
                }
                else if (CycleStyle == 2)
                {
                    loadingLabel.Text += ".";

                    if (loadingLabel.Text.Length > 3)
                    {
                        loadingLabel.Text = "";
                    }
                }
                else
                {
                    loadingLabel.Text = "";
                }

                this.Update();
            }
        }

        public void ExpandForm()
        {
            for (int i = titlePanel.Height; i < 240; i++)
            {
                this.Height = i;

                if ((i % 4) == 0)
                {
                    float opacity = i / 240f;
                    this.Opacity = opacity;

                    this.RepositionForm();

                    System.Threading.Thread.Sleep(10);
                }
            }

            this.Opacity = 1.0f;
            Application.DoEvents();
            this.Update();

            outputListBox.Visible = true;
        }

        public void CollapseForm()
        {
            outputListBox.Visible = false;

            for (int i = 240; i > titlePanel.Height; i--)
            {
                this.Height = i;

                if ((i % 4) == 0)
                {
                    float titleHeight = titlePanel.Height;
                    float opacity = (i / titleHeight) / 10;
                    this.Opacity = opacity;

                    this.RepositionForm();

                    System.Threading.Thread.Sleep(1);
                }
            }

            this.Opacity = 0.0f;
            Application.DoEvents();
        }

        public void RepositionForm()
        {
            this.Top = Screen.PrimaryScreen.WorkingArea.Height - (this.Height + 20);
            this.Left = Screen.PrimaryScreen.WorkingArea.Width - (this.Width + 20);

            outputListBox.Top = 45;
            outputListBox.Left = 15;
            outputListBox.Width = this.Width - 30;
            outputListBox.Height = this.Height - 55;

            Application.DoEvents();
        }

        private void loadTimer_Tick(object sender, EventArgs e)
        {
            loadTimer.Enabled = false;

            this.ExpandForm();

            AddOutput("Initializing...");

            AddOutput("Downloading application schema");
            DownloadFile("\\cache.xml", System.Configuration.ConfigurationManager.AppSettings["applicationDefinitionUrl"], "");

            bool newApplicationFile = false;
            newApplicationFile = !System.IO.File.Exists(AppDomain.CurrentDomain.BaseDirectory + "\\application.xml");

            System.Xml.XmlDocument cacheSchema = new System.Xml.XmlDocument();
            System.Xml.XmlDocument applicationSchema = new System.Xml.XmlDocument();

            cacheSchema.Load(AppDomain.CurrentDomain.BaseDirectory + "\\cache.xml");

            if (!newApplicationFile)
            {
                applicationSchema.Load(AppDomain.CurrentDomain.BaseDirectory + "\\application.xml");
            }

            System.Xml.XmlNodeList cacheNodeList = cacheSchema.DocumentElement.SelectNodes(".//application");

            this.ShowLoadingLabel();

            foreach (System.Xml.XmlNode cacheNode in cacheNodeList)
            {
                Application.DoEvents();

                System.Xml.XmlNodeList fileNodeList = cacheNode.SelectNodes("file");

                foreach (System.Xml.XmlNode fileNode in fileNodeList)
                {
                    bool downloadFile = true;

                    if (!newApplicationFile)
                    {
                        System.Xml.XmlNode applicationFileNode = applicationSchema.DocumentElement.SelectSingleNode("/root/application[@name='" + cacheNode.Attributes["name"].Value + "']/file[@url='" + fileNode.Attributes["url"].Value + "']");

                        if (applicationFileNode != null)
                        {
                            downloadFile = applicationFileNode.Attributes["version"].Value != fileNode.Attributes["version"].Value;
                        }
                    }

                    if (downloadFile)
                    {
                        AddOutput("Downloading file " + fileNode.Attributes["name"].Value);
                        Application.DoEvents();

                        DownloadFile(fileNode.Attributes["name"].Value, fileNode.Attributes["url"].Value, fileNode.Attributes["folder"].Value);
                    }
                }

                int executeCount = 0;
                System.Xml.XmlNode sessionNode = sessionDocument.CreateElement("session");

                foreach (System.Xml.XmlNode applicationSessionNode in sessionDocument.DocumentElement.ChildNodes)
                {
                    if (applicationSessionNode.Attributes["name"].Value == cacheNode.Attributes["name"].Value)
                    {
                        executeCount = applicationSessionNode.ChildNodes.Count;

                        sessionNode.AppendChild(sessionDocument.CreateAttribute("starttime"));

                        sessionNode.Attributes["starttime"].Value = DateTime.Today.ToShortDateString() + " - " + DateTime.Now.ToShortTimeString();

                        applicationSessionNode.AppendChild(sessionNode);
                    }
                }

                if ((cacheNode.Attributes["executeOnce"].Value == "false") ||
                    ((cacheNode.Attributes["executeOnce"].Value == "true") &&
                    (executeCount == 0)))
                {
                    System.Diagnostics.ProcessStartInfo psi;

                    if (cacheNode.Attributes.GetNamedItem("parameters") != null)
                    {
                        psi = new System.Diagnostics.ProcessStartInfo(AppDomain.CurrentDomain.BaseDirectory + cacheNode.Attributes["folder"].Value + "\\" + cacheNode.Attributes["executable"].Value, cacheNode.Attributes["parameters"].Value);
                    }
                    else
                    {
                        psi = new System.Diagnostics.ProcessStartInfo(AppDomain.CurrentDomain.BaseDirectory + cacheNode.Attributes["folder"].Value + "\\" + cacheNode.Attributes["executable"].Value);
                    }

                    System.Diagnostics.Process p = new System.Diagnostics.Process();

                    psi.Verb = cacheNode.Attributes["verb"].Value;

                    p.StartInfo = psi;
                    p.Exited += new EventHandler(p_Exited);

                    processList.Add(p);

                    p.Start();
                }
            }

            this.HideLoadingLabel();

            exitTimer.Enabled = true;

            this.CollapseForm();

            this.Hide();

            System.IO.File.Copy(AppDomain.CurrentDomain.BaseDirectory + "\\cache.xml", AppDomain.CurrentDomain.BaseDirectory + "\\application.xml", true);
            System.IO.File.Delete(AppDomain.CurrentDomain.BaseDirectory + "\\cache.xml");
        }

        public void AddOutput(string Output)
        {
            outputListBox.Items.Add(Output);

            outputListBox.SelectedIndex = outputListBox.Items.Count - 1;

            this.Refresh();
        }

        void p_Exited(object sender, EventArgs e)
        {
            //processList.Remove((System.Diagnostics.Process)sender);

            //if (processList.Count == 0)
            //{
            //    this.Close();
            //}
        }

        protected void DownloadFile(string FileName, string FileUrl, string Folder)
        {
            DialogResult retryResult = DialogResult.Retry;

            while (retryResult == DialogResult.Retry)
            {
                try
                {
                    if (!System.IO.Directory.Exists(AppDomain.CurrentDomain.BaseDirectory + Folder + "\\"))
                    {
                        System.IO.Directory.CreateDirectory(AppDomain.CurrentDomain.BaseDirectory + Folder + "\\");
                    }

                    System.Net.WebRequest webRequest = System.Net.WebRequest.CreateDefault(new Uri(FileUrl));

                    if (webRequest.Proxy.GetProxy(new Uri(FileUrl)).OriginalString != FileUrl)
                    {
                        if ((proxyUsername == string.Empty) && (!proxyShown))
                        {
                            ProxyAuthenticationForm proxyAuthenticationForm = new ProxyAuthenticationForm();

                            if (proxyAuthenticationForm.ShowDialog() == DialogResult.OK)
                            {
                                proxyShown = true;

                                proxyServer = proxyAuthenticationForm.serverAddressTextBox.Text;
                                proxyServerAddress = int.Parse(proxyAuthenticationForm.serverPortTextBox.Text);
                                proxyUsername = proxyAuthenticationForm.usernameTextBox.Text;
                                proxyPassword = proxyAuthenticationForm.passwordTextBox.Text;
                                proxyDomain = proxyAuthenticationForm.domainTextBox.Text;

                                if (proxyAuthenticationForm.rememberMeCheckBox.Checked)
                                {
                                    string proxyDocumentString = "<?xml version=\"1.0\" encoding=\"utf-8\" ?><Settings>";

                                    proxyDocumentString += "<Proxy>";
                                    proxyDocumentString += "<Server>";
                                    proxyDocumentString += "<Address>" + proxyServer + "</Address>";
                                    proxyDocumentString += "<Port>" + proxyServerAddress + "</Port>";
                                    proxyDocumentString += "</Server>";
                                    proxyDocumentString += "<Username>" + proxyUsername + "</Username>";
                                    proxyDocumentString += "<Password>" + proxyPassword + "</Password>";
                                    proxyDocumentString += "<Domain>" + proxyDomain + "</Domain>";
                                    proxyDocumentString += "</Proxy>";

                                    proxyDocumentString += "</Settings>";

                                    System.Xml.XmlDocument settingsDocument = new System.Xml.XmlDocument();

                                    settingsDocument.PreserveWhitespace = true;
                                    settingsDocument.LoadXml(proxyDocumentString);

                                    System.Security.Cryptography.TripleDESCryptoServiceProvider tripleDESCryptoServiceProvider = new System.Security.Cryptography.TripleDESCryptoServiceProvider();

                                    tripleDESCryptoServiceProvider.Key = System.Text.UTF8Encoding.GetEncoding("utf-8").GetBytes("There1nceWa5@ManW1thout@");

                                    TrippleDESDocumentEncryption tripleDESDocumentEncryption = new TrippleDESDocumentEncryption(settingsDocument, tripleDESCryptoServiceProvider);

                                    tripleDESDocumentEncryption.Encrypt("Settings");

                                    if (System.IO.File.Exists(AppDomain.CurrentDomain.BaseDirectory + "\\Settings.xml"))
                                    {
                                        System.IO.File.Delete(AppDomain.CurrentDomain.BaseDirectory + "\\Settings.xml");
                                    }

                                    settingsDocument.Save(AppDomain.CurrentDomain.BaseDirectory + "\\Settings.xml");
                                }
                            }
                        }

                        if (proxyUsername != string.Empty)
                        {
                            webRequest.Proxy.Credentials = new System.Net.NetworkCredential(proxyUsername, proxyPassword);
                        }
                    }

                    System.IO.Stream fileStream = webRequest.GetResponse().GetResponseStream();

                    System.IO.FileStream streamWriter = System.IO.File.Create(AppDomain.CurrentDomain.BaseDirectory + Folder + "\\" + FileName);

                    int value = fileStream.ReadByte();
                    int counter = 0;

                    while (value != -1)
                    {
                        streamWriter.WriteByte((byte)value);

                        value = fileStream.ReadByte();
                        counter++;

                        CycleLoadingLabel(cycleStyle);
                    }

                    streamWriter.Close();
                    fileStream.Close();

                    retryResult = DialogResult.OK;
                }
                catch (System.Net.WebException webException)
                {
                    if (webException.Message == "Unable to connect to the remote server")
                    {
                        string message = "";

                        message = "Your computer does not seem to be connected to the internet!";
                        message += "\r\n\r\n";
                        message += "Please make sure you are dialed up and connected and click retry.";
                        message += "\r\n";
                        message += "If this error continues to appear even after connecting, please contact the ";
                        message += "Eliyahu Technical Support Centre at (012) 123 4567.";

                        retryResult = System.Windows.Forms.MessageBox.Show(
                            message,
                            "Connection Failure",
                            MessageBoxButtons.RetryCancel,
                            MessageBoxIcon.Error,
                            MessageBoxDefaultButton.Button1);

                        proxyUsername = string.Empty;
                        proxyPassword = string.Empty;
                    }
                    else
                    {
                        string message = "";

                        message = "A problem occured while trying to connect to the Eliyahu Web Server. A description of the error follows:";
                        message += "\r\n\r\n";
                        message += webException.Message;
                        message += "\r\n\r\n";
                        message += "Please make sure you are dialed up and connected and click retry.";
                        message += "\r\n";
                        message += "If this error continues to appear even after connecting, please contact the ";
                        message += "Eliyahu Technical Support Centre at (012) 123 4567.";

                        retryResult = System.Windows.Forms.MessageBox.Show(
                            message,
                            "Connection Failure",
                            MessageBoxButtons.RetryCancel,
                            MessageBoxIcon.Error,
                            MessageBoxDefaultButton.Button1);

                        proxyUsername = string.Empty;
                        proxyPassword = string.Empty;
                    }
                }
                catch (System.Exception exception)
                {
                    string message = "";

                    message = "A problem occured while trying to connect to the Eliyahu Web Server. A description of the error follows:";
                    message += "\r\n\r\n";
                    message += exception.Message;
                    message += "\r\n\r\n";
                    message += "Please make sure you are dialed up and connected and click retry.";
                    message += "\r\n";
                    message += "If this error continues to appear even after connecting, please contact the ";
                    message += "Eliyahu Technical Support Centre at (012) 123 4567.";

                    retryResult = System.Windows.Forms.MessageBox.Show(
                        message,
                        "Connection Failure",
                        MessageBoxButtons.RetryCancel,
                        MessageBoxIcon.Error,
                        MessageBoxDefaultButton.Button1);

                    proxyUsername = string.Empty;
                    proxyPassword = string.Empty;
                }
            }
        }

        private void exitTimer_Tick(object sender, EventArgs e)
        {
            System.Collections.ArrayList processes = new System.Collections.ArrayList();

            for (int i = 0; i < processList.Count; i++)
            {
                if (processList[i].HasExited)
                {
                    processes.Add(processList[i]);
                }
            }

            foreach (object o in processes)
            {
                ((System.Diagnostics.Process)o).Close();

                processList.Remove((System.Diagnostics.Process)o);
            }

            if (processList.Count == 0)
            {
                this.Close();
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            try
            {
                exitTimer.Enabled = false;

                System.Collections.ArrayList processes = new System.Collections.ArrayList();

                for (int i = 0; i < processList.Count; i++)
                {
                    processList[i].Close();
                }

                sessionDocument.Save(AppDomain.CurrentDomain.BaseDirectory + "\\session.xml");
            }
            catch
            {
            }
        }
    }
}