#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;
using Enumerators;

namespace BestFit
{
    public class DebtorAccount
    {
        protected int debtorProfileId;
        protected int debtorAccountId;

        protected AccountType accountType;
        protected AccountStatus accountStatus;

        protected DateTime monthDateTime;

        protected decimal weight;

        protected decimal originalContractValue;
        protected int originalContractTerm;
        protected DateTime originalContractDate;
        protected decimal originalInstallmentValue;
        protected decimal currentSettlementValue;
        protected decimal currentInstallmentValue;
        protected decimal currentInterestRate;
        protected InterestType interestType;

        protected decimal interestPayment;
        protected decimal capitalPayment;
        protected decimal feePayment;
        protected decimal creditLifePayment;
        protected decimal totalInterestValue;
        protected decimal totalCapitalValue;
        protected decimal totalFeeValue;
        protected decimal totalCreditLifeValue;
        protected decimal outstandingCapitalValue;
        protected decimal outstandingInterestValue;
        protected decimal outstandingFeeValue;
        protected decimal outstandingCreditLifeValue;

        protected decimal closingBalance;
        protected int month;

        protected bool inDuplum;
        protected bool maxContractInterestBreached;
        protected bool maxInterestBreached;
        protected bool maxFeesBreached;
        protected bool averageInterestLargerThanContract;
        protected bool contractTerm;
        protected bool bestFitTerm;
        protected bool balance;
        protected bool reckless;
        protected bool underAdministration;

        protected bool bondMaxLengthExceeded;
        protected bool carMaxLengthExceeded;
        protected bool retailMaxLengthExceeded;
        protected bool garnishMaxLengthExceeded;

        protected decimal inDuplumValue;
        protected decimal maxContractInterestActualInterestValue;
        protected decimal maxFeesValue;
        protected decimal maxFeesExcessValue;
        protected decimal maxFeesFeeValue;
        protected decimal averageInterestValueToSetAside;
        protected decimal contractTermValue;
        protected decimal bestFitTermValue;
        protected decimal balanceExpectedStatementBalance;
        protected decimal maxInterestValue;

        public DebtorAccount()
        {
            debtorProfileId = -1;
            debtorAccountId = -1;

            weight = 0.00M;

            accountType = AccountType.Unset;
            accountStatus = AccountStatus.Unset;

            monthDateTime = new DateTime(1900, 1, 1);

            originalContractValue = 0.00M;
            originalContractTerm = 0;
            originalContractDate = new DateTime(1900, 1, 1);
            currentSettlementValue = 0.00M;
            currentInstallmentValue = 0.00M;
            originalInstallmentValue = 0.00M;
            currentInterestRate = 0.00M;
            interestType = InterestType.Unset;

            interestPayment = 0.00M;
            capitalPayment = 0.00M;
            feePayment = 0.00M;
            creditLifePayment = 0.00M;
            totalInterestValue = 0.00M;
            totalCapitalValue = 0.00M;
            totalFeeValue = 0.00M;
            totalCreditLifeValue = 0.00M;
            outstandingCapitalValue = 0.00M;
            outstandingInterestValue = 0.00M;
            outstandingFeeValue = 0.00M;
            outstandingCreditLifeValue = 0.00M;

            closingBalance = 0.00M;
            month = 0;

            inDuplum = false;
            maxContractInterestBreached = false;
            maxInterestBreached = false;
            maxFeesBreached = false;
            averageInterestLargerThanContract = false;
            contractTerm = false;
            bestFitTerm = false;
            balance = false;
            reckless = false;
            underAdministration = false;

            bondMaxLengthExceeded = false;
            carMaxLengthExceeded = false;
            retailMaxLengthExceeded = false;
            garnishMaxLengthExceeded = false;
        }

        public decimal Weight
        {
            get { return weight; }
            set { weight = value; }
        }

        public int DebtorProfileId
        {
            get { return debtorProfileId; }
            set { debtorProfileId = value; }
        }

        public int DebtorAccountId
        {
            get { return debtorAccountId; }
            set { debtorAccountId = value; }
        }

        public Enumerators.AccountType AccountType
        {
            get { return accountType; }
            set { accountType = value; }
        }

        public Enumerators.AccountStatus AccountStatus
        {
            get { return accountStatus; }
            set { accountStatus = value; }
        }

        public DateTime MonthDateTime
        {
            get { return monthDateTime; }
            set { monthDateTime = value; }
        }

        public decimal OriginalContractValue
        {
            get { return originalContractValue; }
            set { originalContractValue = value; }
        }

        public int OriginalContractTerm
        {
            get { return originalContractTerm; }
            set { originalContractTerm = value; }
        }

        public DateTime OriginalContractDate
        {
            get { return originalContractDate; }
            set { originalContractDate = value; }
        }

        public decimal CurrentSettlementValue
        {
            get { return currentSettlementValue; }
            set { currentSettlementValue = value; }
        }

        public decimal CurrentInstallmentValue
        {
            get { return currentInstallmentValue; }
            set { currentInstallmentValue = value; }
        }

        public decimal OriginalInstallmentValue
        {
            get { return originalInstallmentValue; }
            set { originalInstallmentValue = value; }
        }

        public decimal CurrentInterestRate
        {
            get { return currentInterestRate; }
            set { currentInterestRate = value; }
        }

        public decimal InterestPayment
        {
            get { return interestPayment; }
            set { interestPayment = value; }
        }

        public decimal CapitalPayment
        {
            get { return capitalPayment; }
            set { capitalPayment = value; }
        }

        public decimal FeePayment
        {
            get { return feePayment; }
            set { feePayment = value; }
        }

        public decimal CreditLifePayment
        {
            get { return creditLifePayment; }
            set { creditLifePayment = value; }
        }

        public decimal TotalInterestValue
        {
            get { return totalInterestValue; }
            set { totalInterestValue = value; }
        }

        public decimal TotalCapitalValue
        {
            get { return totalCapitalValue; }
            set { totalCapitalValue = value; }
        }

        public decimal TotalFeeValue
        {
            get { return totalFeeValue; }
            set { totalFeeValue = value; }
        }

        public decimal TotalCreditLifeValue
        {
            get { return totalCreditLifeValue; }
            set { totalCreditLifeValue = value; }
        }

        public decimal TotalPaidToDate
        {
            get { return totalCapitalValue + totalCreditLifeValue + totalFeeValue + totalInterestValue; }
        }

        public decimal OutstandingInterestValue
        {
            get { return outstandingInterestValue; }
            set { outstandingInterestValue = value; }
        }

        public decimal OutstandingCapitalValue
        {
            get { return outstandingCapitalValue; }
            set { outstandingCapitalValue = value; }
        }

        public decimal OutstandingFeeValue
        {
            get { return outstandingFeeValue; }
            set { outstandingFeeValue = value; }
        }

        public decimal OutstandingCreditLifeValue
        {
            get { return outstandingCreditLifeValue; }
            set { outstandingCreditLifeValue = value; }
        }

        public decimal ClosingBalance
        {
            get { return closingBalance; }
            set { closingBalance = value; }
        }

        public int Month
        {
            get { return month; }
            set { month = value; }
        }

        public bool InDuplum
        {
            get { return inDuplum; }
            set { inDuplum = value; }
        }

        public bool BondMaxLengthExceeded
        {
            get { return bondMaxLengthExceeded; }
            set { bondMaxLengthExceeded = value; }
        }

        public bool CarMaxLengthExceeded
        {
            get { return carMaxLengthExceeded; }
            set { carMaxLengthExceeded = value; }
        }

        public bool RetailMaxLengthExceeded
        {
            get { return retailMaxLengthExceeded; }
            set { retailMaxLengthExceeded = value; }
        }

        public bool GarnishMaxLengthExceeded
        {
            get { return garnishMaxLengthExceeded; }
            set { garnishMaxLengthExceeded = value; }
        }

        public bool MaxContractInterestBreached
        {
            get { return maxContractInterestBreached; }
            set { maxContractInterestBreached = value; }
        }

        public bool MaxInterestBreached
        {
            get { return maxInterestBreached; }
            set { maxInterestBreached = value; }
        }

        public bool MaxFeesBreached
        {
            get { return maxFeesBreached; }
            set { maxFeesBreached = value; }
        }

        public bool AverageInterestLargerThanContract
        {
            get { return averageInterestLargerThanContract; }
            set { averageInterestLargerThanContract = value; }
        }

        public bool ContractTerm
        {
            get { return contractTerm; }
            set { contractTerm = value; }
        }

        public bool BestFitTerm
        {
            get { return bestFitTerm; }
            set { bestFitTerm = value; }
        }

        public bool Balance
        {
            get { return balance; }
            set { balance = value; }
        }

        public bool Reckless
        {
            get { return reckless; }
            set { reckless = value; }
        }

        public bool UnderAdministration
        {
            get { return underAdministration; }
            set { underAdministration = value; }
        }

        public decimal InDuplumValue
        {
            get
            {
                return inDuplumValue;
            }

            set
            {
                inDuplumValue = value;
            }
        }

        public decimal MaxContractInterestActualInterestValue
        {
            get
            {
                return maxContractInterestActualInterestValue;
            }

            set
            {
                maxContractInterestActualInterestValue = value;
            }
        }

        public decimal MaxFeesValue
        {
            get
            {
                return maxFeesValue;
            }

            set
            {
                maxFeesValue = value;
            }
        }

        public decimal MaxFeesExcessValue
        {
            get
            {
                return maxFeesExcessValue;
            }

            set
            {
                maxFeesExcessValue = value;
            }
        }

        public decimal MaxFeesFeeValue
        {
            get
            {
                return maxFeesFeeValue;
            }

            set
            {
                maxFeesFeeValue = value;
            }
        }

        public decimal AverageInterestValueToSetAside
        {
            get
            {
                return averageInterestValueToSetAside;
            }

            set
            {
                averageInterestValueToSetAside = value;
            }
        }

        public decimal ContractTermValue
        {
            get
            {
                return contractTermValue;
            }

            set
            {
                contractTermValue = value;
            }
        }

        public decimal BestFitTermValue
        {
            get
            {
                return bestFitTermValue;
            }

            set
            {
                bestFitTermValue = value;
            }
        }

        public decimal BalanceExpectedStatementBalance
        {
            get
            {
                return balanceExpectedStatementBalance;
            }

            set
            {
                balanceExpectedStatementBalance = value;
            }
        }

        public decimal MaxInterestValue
        {
            get
            {
                return maxInterestValue;
            }

            set
            {
                maxInterestValue = value;
            }
        }
    }
}
