#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;

namespace Validators
{
    public class MaxContractInterestBreached: ValidatorBase
    {
        public MaxContractInterestBreached(Dictionary<string, string> GeneralSettings)
            : base(GeneralSettings)
        {
        }

        public decimal CalculateActualInterest(decimal ContractValue, decimal TotalInterestValue, decimal InterestRate, DateTime ContractStartDate)
        {
            TimeSpan contractLengthMonthsTimeSpan = new TimeSpan(DateTime.Now.Ticks - ContractStartDate.Ticks);
            int contractLengthMonths = System.Convert.ToInt32(contractLengthMonthsTimeSpan.TotalDays / 30.6667);

            if (contractLengthMonths == 0)
            {
                return 0.00M;
            }
            else
            {
                return (((TotalInterestValue / ContractValue) / contractLengthMonths) * 12);
            }
        }

        public bool Validate(decimal ContractValue, decimal TotalInterestValue, decimal InterestRate, DateTime ContractStartDate)
        {
            bool resultValue = false;

            resultValue = CalculateActualInterest(ContractValue, TotalInterestValue, InterestRate, ContractStartDate) > InterestRate;

            return resultValue;
        }

        public override bool Validate()
        {
            throw new Exception("Use the Validate(decimal ContractValue, decimal TotalInterestValue, decimal InterestRate, DateTime ContractStartDate) overload method.");
        }
    }
}
