#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;

namespace Validators
{
    public class MaxFeesBreached: ValidatorBase
    {
        protected decimal maxFees;
        protected decimal maxFeesExcess;
        protected decimal fees;

        public MaxFeesBreached(Dictionary<string, string> GeneralSettings): base(GeneralSettings)
        {
        }

        protected decimal calculateMaxFees(decimal OriginalContractValue, string BaseFeeCode, string PercentageCode, string ExcessCode)
        {
            decimal resultValue = 0.00M;

            decimal baseFee = getDecimalValue(BaseFeeCode);
            decimal percentage = getDecimalValue(PercentageCode);
            decimal excessValue = getDecimalValue(ExcessCode);

            decimal percentageValue = 0.00M;

            if (OriginalContractValue > excessValue)
            {
                percentageValue = OriginalContractValue * percentage;
            }

            resultValue += baseFee;
            resultValue += percentageValue;

            maxFees = resultValue;

            return resultValue;
        }

        protected decimal calculateExcessFees(decimal OriginalContractValue, string BaseFeeCode, string PercentageCode, string ExcessCode, string LimitCode)
        {
            decimal resultValue = 0.00M;

            decimal limitValue = getDecimalValue(LimitCode);
            decimal feeValue = calculateMaxFees(OriginalContractValue, BaseFeeCode, PercentageCode, ExcessCode);

            if (feeValue > limitValue)
            {
                resultValue = feeValue - limitValue;
            }

            maxFeesExcess = resultValue;

            return resultValue;
        }

        protected decimal calculateFees(decimal OriginalContractValue, string BaseFeeCode, string PercentageCode, string ExcessCode, string LimitCode)
        {
            decimal resultValue = 0.00M;

            decimal limitValue = getDecimalValue(LimitCode);
            decimal feeValue = calculateMaxFees(OriginalContractValue, BaseFeeCode, PercentageCode, ExcessCode);

            if (feeValue > limitValue)
            {
                resultValue = limitValue;
            }
            else
            {
                resultValue = feeValue;
            }

            fees = resultValue;

            return resultValue;
        }

        public decimal CalculateMaxFeesForCredit(decimal OriginalContractValue)
        {
            return calculateFees(OriginalContractValue, "BFCREDIT", "PCCREDIT", "ECCREDIT", "LCCREDIT");
        }

        public decimal CalculateMaxFeesForDevelopmentLowIncome(decimal OriginalContractValue)
        {
            return calculateFees(OriginalContractValue, "BFDLOWINCOME", "PCDLOWINCOME", "ECDLOWINCOME", "LCDLOWINCOME");
        }

        public decimal CalculateMaxFeesForDevelopmentSmallBusiness(decimal OriginalContractValue)
        {
            return calculateFees(OriginalContractValue, "BFDSMALLBUSINESS", "PCDSMALLBUSINESS", "ECDSMALLBUSINESS", "LCDSMALLBUSINESS");
        }

        public decimal CalculateMaxFeesForIncidental(decimal OriginalContractValue)
        {
            return 0.00M; // calculateFees(OriginalContractValue, "BFINCIDENTAL", "PCINCIDENTAL", "ECINCIDENTAL", "LCINCIDENTAL");
        }

        public decimal CalculateMaxFeesForMortage(decimal OriginalContractValue)
        {
            return calculateFees(OriginalContractValue, "BFMORTAGE", "PCMORTAGE", "ECMORTAGE", "LCMORTAGE");
        }

        public decimal CalculateMaxFeesForOther(decimal OriginalContractValue)
        {
            return calculateFees(OriginalContractValue, "BFOTHER", "PCOTHER", "ECOTHER", "LCOTHER");
        }

        public decimal CalculateMaxFeesForShortTerm(decimal OriginalContractValue)
        {
            return calculateFees(OriginalContractValue, "BFSHORTTERM", "PCSHORTTERM", "ECSHORTTERM", "LCSHORTTERM");
        }

        public decimal CalculateMaxFeesForUnsecureCredit(decimal OriginalContractValue)
        {
            return calculateFees(OriginalContractValue, "BFUNSECURECREDIT", "PCUNSECURECREDIT", "ECUNSECURECREDIT", "LCUNSECURECREDIT");
        }

        public decimal CalculateMaxFeesForUnset(decimal OriginalContractValue)
        {
            return calculateFees(OriginalContractValue, "BFUNSET", "PCUNSET", "ECUNSET", "LCUNSET");
        }

        public bool Validate(decimal OriginalContractValue, decimal TotalFeesPaid, Enumerators.AccountType AccountType)
        {
            bool resultValue = false;

            switch (AccountType)
            {
                //case Enumerators.AccountType.Credit:
                //    resultValue = CalculateMaxFeesForCredit(OriginalContractValue) < TotalFeesPaid;
                //    break;
                //case Enumerators.AccountType.DevelopmentLowIncome:
                //    resultValue = CalculateMaxFeesForDevelopmentLowIncome(OriginalContractValue) < TotalFeesPaid;
                //    break;
                //case Enumerators.AccountType.DevelopmentSmallBusiness:
                //    resultValue = CalculateMaxFeesForDevelopmentSmallBusiness(OriginalContractValue) < TotalFeesPaid;
                //    break;
                //case Enumerators.AccountType.Incidental:
                //    resultValue = CalculateMaxFeesForIncidental(OriginalContractValue) < TotalFeesPaid;
                //    break;
                //case Enumerators.AccountType.Mortage:
                //    resultValue = CalculateMaxFeesForMortage(OriginalContractValue) < TotalFeesPaid;
                //    break;
                //case Enumerators.AccountType.Other:
                //    resultValue = CalculateMaxFeesForOther(OriginalContractValue) < TotalFeesPaid;
                //    break;
                //case Enumerators.AccountType.ShortTerm:
                //    resultValue = CalculateMaxFeesForShortTerm(OriginalContractValue) < TotalFeesPaid;
                //    break;
                //case Enumerators.AccountType.UnsecureCredit:
                //    resultValue = CalculateMaxFeesForUnsecureCredit(OriginalContractValue) < TotalFeesPaid;
                //    break;
                //case Enumerators.AccountType.Unset:
                //    resultValue = CalculateMaxFeesForUnset(OriginalContractValue) < TotalFeesPaid;
                //    break;
                default:
                    break;
            }

            return resultValue;
        }

        public override bool Validate()
        {
            throw new Exception("Use the Validate(decimal OriginalContractValue, decimal TotalFeesPaid, Enumerators.AccountType AccountType) overload method.");
        }

        public decimal MaxFees
        {
            get
            {
                return maxFees;
            }

            set
            {
                maxFees = value;
            }
        }

        public decimal MaxFeesExcess
        {
            get
            {
                return maxFeesExcess;
            }

            set
            {
                maxFeesExcess = value;
            }
        }

        public decimal Fees
        {
            get
            {
                return fees;
            }

            set
            {
                fees = value;
            }
        }
    }
}
