#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;

namespace Validators
{
    public class MaxInterestBreached: ValidatorBase
    {
        protected decimal maxInterest;

        public MaxInterestBreached(Dictionary<string, string> GeneralSettings)
            : base(GeneralSettings)
        {
        }

        protected decimal CalculateMaxInterest(decimal OriginalContractValue, int ContractLength, int ContractTerm, string CreditCode)
        {
            decimal repoRate = getDecimalValue("REPORATE");
            decimal repoRateMultiplier = getDecimalValue("REPORATEMULTIPLIER");
            decimal annualPercentage = getDecimalValue(CreditCode);

            decimal percentageValue = (OriginalContractValue * annualPercentage) / 12;
            percentageValue = percentageValue * ContractLength;

            maxInterest = (repoRate * repoRateMultiplier) + percentageValue;

            return maxInterest;
        }

        public decimal CalculateMaxInterestForCredit(decimal OriginalContractValue, int ContractLength, int ContractTerm)
        {
            return CalculateMaxInterest(OriginalContractValue, ContractLength, ContractTerm, "APCREDIT");
        }

        public decimal CalculateMaxInterestForDevelopmentLowIncome(decimal OriginalContractValue, int ContractLength, int ContractTerm)
        {
            return CalculateMaxInterest(OriginalContractValue, ContractLength, ContractTerm, "APDLOWINCOME");
        }

        public decimal CalculateMaxInterestForDevelopmentSmallBusiness(decimal OriginalContractValue, int ContractLength, int ContractTerm)
        {
            return CalculateMaxInterest(OriginalContractValue, ContractLength, ContractTerm, "APDSMALLBUSINESS");
        }

        public decimal CalculateMaxInterestForIncidental(decimal OriginalContractValue, int ContractLength, int ContractTerm)
        {
            return CalculateMaxInterest(OriginalContractValue, ContractLength, ContractTerm, "APINCIDENTAL");
        }

        public decimal CalculateMaxInterestForMortage(decimal OriginalContractValue, int ContractLength, int ContractTerm)
        {
            return CalculateMaxInterest(OriginalContractValue, ContractLength, ContractTerm, "APMORTAGE");
        }

        public decimal CalculateMaxInterestForOther(decimal OriginalContractValue, int ContractLength, int ContractTerm)
        {
            return CalculateMaxInterest(OriginalContractValue, ContractLength, ContractTerm, "APOTHER");
        }

        public decimal CalculateMaxInterestForShortTerm(decimal OriginalContractValue, int ContractLength, int ContractTerm)
        {
            return CalculateMaxInterest(OriginalContractValue, ContractLength, ContractTerm, "APSHORTTERM");
        }

        public decimal CalculateMaxInterestForUnsecureCredit(decimal OriginalContractValue, int ContractLength, int ContractTerm)
        {
            return CalculateMaxInterest(OriginalContractValue, ContractLength, ContractTerm, "APUNSECURECREDIT");
        }

        public decimal CalculateMaxInterestForUnset(decimal OriginalContractValue, int ContractLength, int ContractTerm)
        {
            return CalculateMaxInterest(OriginalContractValue, ContractLength, ContractTerm, "APUNSET");
        }

        public bool Validate(decimal TotalInterestPaid, decimal OriginalContractValue, int ContractLength, int ContractTerm, Enumerators.AccountType AccountType)
        {
            bool resultValue = false;

            switch (AccountType)
            {
                //case Enumerators.AccountType.Credit:
                //    resultValue = CalculateMaxInterestForCredit(OriginalContractValue, ContractLength, ContractTerm) < TotalInterestPaid;
                //    break;
                //case Enumerators.AccountType.DevelopmentLowIncome:
                //    resultValue = CalculateMaxInterestForDevelopmentLowIncome(OriginalContractValue, ContractLength, ContractTerm) < TotalInterestPaid;
                //    break;
                //case Enumerators.AccountType.DevelopmentSmallBusiness:
                //    resultValue = CalculateMaxInterestForDevelopmentSmallBusiness(OriginalContractValue, ContractLength, ContractTerm) < TotalInterestPaid;
                //    break;
                //case Enumerators.AccountType.Incidental:
                //    resultValue = CalculateMaxInterestForIncidental(OriginalContractValue, ContractLength, ContractTerm) < TotalInterestPaid;
                //    break;
                //case Enumerators.AccountType.Mortage:
                //    resultValue = CalculateMaxInterestForMortage(OriginalContractValue, ContractLength, ContractTerm) < TotalInterestPaid;
                //    break;
                //case Enumerators.AccountType.Other:
                //    resultValue = CalculateMaxInterestForOther(OriginalContractValue, ContractLength, ContractTerm) < TotalInterestPaid;
                //    break;
                //case Enumerators.AccountType.ShortTerm:
                //    resultValue = CalculateMaxInterestForShortTerm(OriginalContractValue, ContractLength, ContractTerm) < TotalInterestPaid;
                //    break;
                //case Enumerators.AccountType.UnsecureCredit:
                //    resultValue = CalculateMaxInterestForUnsecureCredit(OriginalContractValue, ContractLength, ContractTerm) < TotalInterestPaid;
                //    break;
                //case Enumerators.AccountType.Unset:
                //    resultValue = CalculateMaxInterestForUnset(OriginalContractValue, ContractLength, ContractTerm) < TotalInterestPaid;
                //    break;
                default:
                    break;
            }

            return resultValue;
        }

        public override bool Validate()
        {
            throw new Exception("Use the Validate(decimal TotalInterestPaid, decimal interestPayment) overload method.");
        }

        public decimal MaxInterest
        {
            get
            {
                return maxInterest;
            }

            set
            {
                maxInterest = value;
            }
        }
    }
}
