#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Text;

namespace Validators
{
    public class ValidatorFactory
    {
        #region Validate
        public static bool InDuplum(Dictionary<string, string> GeneralSettings, Decimal ContractValue, decimal TotalInterestValue)
        {
            InDuplum inDuplum = new InDuplum(GeneralSettings);

            return inDuplum.Validate(ContractValue, TotalInterestValue);
        }

        public static bool AverageInterestLargerThanContract(Dictionary<string, string> GeneralSettings, decimal InterestRate, int OriginalContractTerm, decimal OriginalContractValue)
        {
            AverageInterestLargerThanContract averageInterestLargerThanContract = new AverageInterestLargerThanContract(GeneralSettings);

            return averageInterestLargerThanContract.Validate(InterestRate, OriginalContractTerm, OriginalContractValue);
        }

        public static bool MaxContractInterestBreached(Dictionary<string, string> GeneralSettings, decimal ContractValue, decimal TotalInterestValue, decimal InterestRate, DateTime ContractStartDate)
        {
            MaxContractInterestBreached maxContractInterestBreached = new MaxContractInterestBreached(GeneralSettings);

            return maxContractInterestBreached.Validate(ContractValue, TotalInterestValue, InterestRate, ContractStartDate);
        }

        public static bool MaxInterestBreached(Dictionary<string, string> GeneralSettings, decimal TotalInterestPaid, decimal OriginalContractValue, int ContractLength, int ContractTerm, Enumerators.AccountType AccountType)
        {
            MaxInterestBreached maxInterestBreached = new MaxInterestBreached(GeneralSettings);

            return maxInterestBreached.Validate(TotalInterestPaid, OriginalContractValue, ContractLength, ContractTerm, AccountType);
        }

        public static bool MaxFeesBreached(Dictionary<string, string> GeneralSettings, decimal OriginalContractValue, decimal TotalFeesPaid, Enumerators.AccountType AccountType)
        {
            MaxFeesBreached maxFeesBreached = new MaxFeesBreached(GeneralSettings);

            return maxFeesBreached.Validate(OriginalContractValue, TotalFeesPaid, AccountType);
        }

        public static bool ContractTerm(Dictionary<string, string> GeneralSettings, DateTime ContractStartDate, int ContractTerm)
        {
            ContractTerm contractTerm = new ContractTerm(GeneralSettings);

            return contractTerm.Validate(ContractStartDate, ContractTerm);
        }

        public static bool BestFitTerm(Dictionary<string, string> GeneralSettings, int CurrentMonth, DateTime ContractStartDate, int ContractTerm)
        {
            BestFitTerm bestFitTerm = new BestFitTerm(GeneralSettings);

            return bestFitTerm.Validate(CurrentMonth, ContractStartDate, ContractTerm);
        }

        public static bool Balance(Dictionary<string, string> GeneralSettings, decimal OriginalContractValue, decimal CreditLifePayment, int ContractLength, decimal TotalInterestPayment, decimal TotalPaidToDate, decimal ClosingBalance)
        {
            Balance balance = new Balance(GeneralSettings);

            return balance.Validate(OriginalContractValue, CreditLifePayment, ContractLength, TotalInterestPayment, TotalPaidToDate, ClosingBalance);
        }

        public static bool UnderAdministration(Dictionary<string, string> GeneralSettings, Enumerators.AdministrationType AdministrationType)
        {
            UnderAdministration underAdministration = new UnderAdministration(GeneralSettings);

            return underAdministration.Validate(AdministrationType);
        }

        public static bool BondMaxLengthExceeded(Dictionary<string, string> GeneralSettings, DateTime ContractStartDate)
        {
            BondMaxLengthExceeded bondMaxLengthExceeded = new BondMaxLengthExceeded(GeneralSettings);

            return bondMaxLengthExceeded.Validate(ContractStartDate);
        }

        public static bool CarMaxLengthExceeded(Dictionary<string, string> GeneralSettings, DateTime ContractStartDate)
        {
            CarMaxLengthExceeded carMaxLengthExceeded = new CarMaxLengthExceeded(GeneralSettings);

            return carMaxLengthExceeded.Validate(ContractStartDate);
        }

        public static bool RetailMaxLengthExceeded(Dictionary<string, string> GeneralSettings, DateTime ContractStartDate)
        {
            RetailMaxLengthExceeded retailMaxLengthExceeded = new RetailMaxLengthExceeded(GeneralSettings);

            return retailMaxLengthExceeded.Validate(ContractStartDate);
        }

        public static bool GarnishMaxLengthExceeded(Dictionary<string, string> GeneralSettings, DateTime ContractStartDate)
        {
            GarnishMaxLengthExceeded garnishMaxLengthExceeded = new GarnishMaxLengthExceeded(GeneralSettings);

            return garnishMaxLengthExceeded.Validate(ContractStartDate);
        }
        #endregion

        #region Calculations
        public static decimal InDuplumValue(Dictionary<string, string> GeneralSettings, Decimal ContractValue, decimal TotalInterestValue)
        {
            InDuplum inDuplum = new InDuplum(GeneralSettings);

            return inDuplum.CalculateInDuplum(ContractValue, TotalInterestValue);
        }

        public static decimal MaxContractInterestBreachedValue(Dictionary<string, string> GeneralSettings, decimal ContractValue, decimal TotalInterestValue, decimal InterestRate, DateTime ContractStartDate)
        {
            MaxContractInterestBreached maxContractInterestBreached = new MaxContractInterestBreached(GeneralSettings);

            return maxContractInterestBreached.CalculateActualInterest(ContractValue, TotalInterestValue, InterestRate, ContractStartDate);
        }

        public static decimal MaxInterestBreachedValue(Dictionary<string, string> GeneralSettings, decimal TotalInterestPaid, decimal OriginalContractValue, int ContractLength, int ContractTerm, Enumerators.AccountType AccountType)
        {
            MaxInterestBreached maxInterestBreached = new MaxInterestBreached(GeneralSettings);

            maxInterestBreached.Validate(TotalInterestPaid, OriginalContractValue, ContractLength, ContractTerm, AccountType);

            return maxInterestBreached.MaxInterest;
        }

        public static decimal MaxFeesBreachedValue(Dictionary<string, string> GeneralSettings, decimal OriginalContractValue, decimal TotalFeesPaid, Enumerators.AccountType AccountType)
        {
            MaxFeesBreached maxFeesBreached = new MaxFeesBreached(GeneralSettings);

            maxFeesBreached.Validate(OriginalContractValue, TotalFeesPaid, AccountType);

            return maxFeesBreached.MaxFees;
        }

        public static decimal MaxFeesBreachedExcessValue(Dictionary<string, string> GeneralSettings, decimal OriginalContractValue, decimal TotalFeesPaid, Enumerators.AccountType AccountType)
        {
            MaxFeesBreached maxFeesBreached = new MaxFeesBreached(GeneralSettings);

            maxFeesBreached.Validate(OriginalContractValue, TotalFeesPaid, AccountType);

            return maxFeesBreached.MaxFeesExcess;
        }

        public static decimal MaxFeesBreachedFeeValue(Dictionary<string, string> GeneralSettings, decimal OriginalContractValue, decimal TotalFeesPaid, Enumerators.AccountType AccountType)
        {
            MaxFeesBreached maxFeesBreached = new MaxFeesBreached(GeneralSettings);

            maxFeesBreached.Validate(OriginalContractValue, TotalFeesPaid, AccountType);

            return maxFeesBreached.Fees;
        }

        public static decimal AverageInterestLargerThanContractToSetAside(Dictionary<string, string> GeneralSettings, decimal InterestRate, int OriginalContractTerm, decimal OriginalContractValue)
        {
            AverageInterestLargerThanContract averageInterestLargerThanContract = new AverageInterestLargerThanContract(GeneralSettings);

            return averageInterestLargerThanContract.CalculateValueToSetAside(InterestRate, OriginalContractTerm, OriginalContractValue);
        }

        public static decimal ContractTermLength(Dictionary<string, string> GeneralSettings, DateTime ContractStartDate, int ContractTerm)
        {
            ContractTerm contractTerm = new ContractTerm(GeneralSettings);

            return contractTerm.CalculateCurrentContractLength(ContractStartDate);
        }

        public static decimal BestFitTermLength(Dictionary<string, string> GeneralSettings, int CurrentMonth, DateTime ContractStartDate, int ContractTerm)
        {
            BestFitTerm bestFitTerm = new BestFitTerm(GeneralSettings);

            return bestFitTerm.CalculateBestFitLength(bestFitTerm.CalculateCurrentContractLength(ContractStartDate), CurrentMonth);
        }

        public static decimal BalanceExpectedStatementValue(Dictionary<string, string> GeneralSettings, decimal OriginalContractValue, decimal CreditLifePayment, int ContractLength, decimal TotalInterestPayment, decimal TotalPaidToDate, decimal ClosingBalance)
        {
            Balance balance = new Balance(GeneralSettings);

            return balance.CalculateExpectedStatementBalance(OriginalContractValue, CreditLifePayment, ContractLength, TotalInterestPayment, TotalPaidToDate);
        }
        #endregion
    }
}
