#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Web;
using System.Collections;
using System.Web.Services;
using System.Web.Services.Protocols;


/// <summary>
/// Summary description for FileManager
/// </summary>
[WebService(Namespace = "http://genesis.bluemarble.co.za/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
public class FileManager : WebServiceBase
{

    public FileManager()
    {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    [WebMethod]
    public Guid UploadFile(string DocumentType, string FileName, string FileType, int FileSize, Guid UserGuid)
    {
        Guid returnValue = Guid.NewGuid();

        try
        {
            int documentTypeId = 0;

            Lookup lookup = new Lookup();

            System.Data.DataTable documentTypeTable = lookup.GetLookupValuesUsingConnection("[Lookup].[DocumentType]", "DocumentTypeId", "DocumentTypeName", "EliyahuLiteConnectionString");

            System.Data.DataRow[] documentTypeRows = documentTypeTable.Select("DocumentTypeName = '" + DocumentType + "'");

            if (documentTypeRows.Length > 0)
            {
                documentTypeId = System.Convert.ToInt32(documentTypeRows[0]["DocumentTypeId"]);
            }

            string commandText = "INSERT INTO System.Document (DocumentGuid, DocumentTypeId, FileType, FileName, FileSize, DocumentUrl, UserGuid) VALUES (@DocumentGuid, @DocumentTypeId, @FileType, @FileName, @FileSize, @DocumentUrl, @UserGuid)";

            System.Data.CommandType commandType = System.Data.CommandType.Text;

            connectionStringKey = "EliyahuLiteConnectionString";

            OpenSqlConnection();

            GetSqlCommand(commandText, commandType).Parameters.Add("@DocumentGuid", returnValue);
            GetSqlCommand(commandText, commandType).Parameters.Add("@DocumentTypeId", documentTypeId);
            GetSqlCommand(commandText, commandType).Parameters.Add("@FileType", FileType);
            GetSqlCommand(commandText, commandType).Parameters.Add("@FileName", FileName.Substring(FileName.LastIndexOf("\\") + 1, FileName.Length - (FileName.LastIndexOf("\\") + 1)));
            GetSqlCommand(commandText, commandType).Parameters.Add("@FileSize", FileSize);
            GetSqlCommand(commandText, commandType).Parameters.Add("@DocumentUrl", this.SetUrlPath(returnValue) + returnValue.ToString() + "." + FileType);
            GetSqlCommand(commandText, commandType).Parameters.Add("@UserGuid", UserGuid);

            ExecuteNonQuery(commandText);

            ResetCommandParameters();

            CloseSqlConnection();
        }
        catch (System.Exception e)
        {
            returnValue = Guid.Empty;
        }

        return returnValue;
    }

    [WebMethod]
    public Guid UploadDebtorDocument(Guid DocumentGuid, int DebtorProfileId, int DebtorAccountId)
    {
        Guid returnValue = Guid.Empty;

        try
        {
            string commandText = "INSERT INTO Debtor.DebtorDocument (DocumentGuid, DebtorProfileId, DebtorAccountId) VALUES (@DocumentGuid, @DebtorProfileId, @DebtorAccountId)";

            System.Data.CommandType commandType = System.Data.CommandType.Text;

            connectionStringKey = "EliyahuLiteConnectionString";

            OpenSqlConnection();

            if (DebtorProfileId != -1)
            {
                GetSqlCommand(commandText, commandType).Parameters.Add("@DebtorProfileId", DebtorProfileId);

                if (DebtorAccountId != -1)
                {
                    GetSqlCommand(commandText, commandType).Parameters.Add("@DebtorAccountId", DebtorAccountId);
                }
                else
                {
                    GetSqlCommand(commandText, commandType).Parameters.Add("@DebtorAccountId", DBNull.Value);
                }
            }

            GetSqlCommand(commandText, commandType).Parameters.Add("@DocumentGuid", DocumentGuid);

            ExecuteNonQuery(commandText);

            ResetCommandParameters();

            commandText = "SELECT DebtorDocumentGuid FROM Debtor.DebtorDocument WHERE DebtorProfileId = @DebtorProfileId AND DocumentGuid = @DocumentGuid ";

            commandType = System.Data.CommandType.Text;

            connectionStringKey = "EliyahuLiteConnectionString";

            OpenSqlConnection();

            if (DebtorProfileId != -1)
            {
                GetSqlCommand(commandText, commandType).Parameters.Add("@DebtorProfileId", DebtorProfileId);
            }

            GetSqlCommand(commandText, commandType).Parameters.Add("@DocumentGuid", DocumentGuid);

            returnValue = (Guid)ExecuteScalar(commandText);

            ResetCommandParameters();

            CloseSqlConnection();
        }
        catch (System.Exception e)
        {
            returnValue = Guid.Empty;
        }

        return returnValue;
    }

    [WebMethod]
    public Guid UploadSubscriberLogo(string FileName, string FileType, int FileSize, int SubscriberConfigurationId)
    {
        Guid returnValue = Guid.NewGuid();

        try
        {
            string commandText = "UPDATE System.SubscriberConfiguration SET LawyerLogo = @LawyerLogo WHERE SubscriberConfigurationId = @SubscriberConfigurationId ";

            System.Data.CommandType commandType = System.Data.CommandType.Text;

            connectionStringKey = "EliyahuLiteConnectionString";

            OpenSqlConnection();

            string lawyerLogoUrl = SetUrlPath(returnValue) + returnValue.ToString() + "." + FileType;

            GetSqlCommand(commandText, commandType).Parameters.Add("@LawyerLogo", lawyerLogoUrl);
            GetSqlCommand(commandText, commandType).Parameters.Add("@SubscriberConfigurationId", SubscriberConfigurationId);

            ExecuteNonQuery(commandText);

            ResetCommandParameters();

            CloseSqlConnection();
        }
        catch (System.Exception e)
        {
            returnValue = Guid.Empty;
        }

        return returnValue;
    }

    [WebMethod]
    public Guid UploadSourceLogo(string FileName, string FileType, int FileSize, int SourceConfigurationId)
    {
        Guid returnValue = Guid.NewGuid();

        try
        {
            string commandText = "UPDATE System.SourceConfiguration SET SourceLogo = @SourceLogo WHERE SourceConfigurationId = @SourceConfigurationId ";

            System.Data.CommandType commandType = System.Data.CommandType.Text;

            connectionStringKey = "EliyahuLiteConnectionString";

            OpenSqlConnection();

            string sourceLogoUrl = SetUrlPath(returnValue) + returnValue.ToString() + "." + FileType;

            GetSqlCommand(commandText, commandType).Parameters.Add("@SourceLogo", sourceLogoUrl);
            GetSqlCommand(commandText, commandType).Parameters.Add("@SourceConfiguration", SourceConfigurationId);

            ExecuteNonQuery(commandText);

            ResetCommandParameters();

            CloseSqlConnection();
        }
        catch (System.Exception e)
        {
            returnValue = Guid.Empty;
        }

        return returnValue;
    }

    [WebMethod]
    public void UploadFilePart(Guid DocumentGuid, int FilePartNumber, byte[] FilePart)
    {
        string filePath = SetFilePath(DocumentGuid);

        System.IO.FileStream fs = System.IO.File.Create(filePath + "temporary\\" + DocumentGuid.ToString() + "." + FilePartNumber.ToString());

        fs.Write(FilePart, 0, FilePart.Length);

        fs.Close();
    }

    [WebMethod]
    public void JoinFileParts(Guid DocumentGuid)
    {
        string filePath = SetFilePath(DocumentGuid);

        System.IO.FileStream fileWriter = System.IO.File.Create(filePath + DocumentGuid.ToString() + "." + GetFileType(DocumentGuid));

        string[] files = System.IO.Directory.GetFiles(filePath + "temporary\\", DocumentGuid.ToString() + ".*");

        long runningFileSize = 0;
        for (int i = 0; i < files.Length; i++)
        {
            string file = files[i];

            System.IO.FileInfo fi = new System.IO.FileInfo(file);

            runningFileSize += fi.Length;
        }

        byte[] blankFile = new byte[runningFileSize];

        for (int i = 0; i < runningFileSize; i++)
        {
            blankFile[i] = 0;
        }

        fileWriter.Write(blankFile, 0, System.Convert.ToInt32(runningFileSize));
        fileWriter.Close();

        fileWriter = System.IO.File.OpenWrite(filePath + DocumentGuid.ToString() + "." + GetFileType(DocumentGuid)); 

        for (int i = 0; i < files.Length; i++)
        {
            string file = files[i];

            System.IO.FileStream fileReader = System.IO.File.OpenRead(filePath + "temporary\\" + DocumentGuid.ToString() + "." + i.ToString());

            byte[] fileContents = System.IO.File.ReadAllBytes(file);

            fileWriter.Write(fileContents, 0, fileContents.Length);
        }

        fileWriter.Close();

        //System.IO.Directory.Delete(filePath + "temporary\\", true);
    }

    [WebMethod]
    public void DeleteTemporaryFolder(Guid DocumentGuid)
    {
        string filePath = SetFilePath(DocumentGuid);

        System.IO.Directory.Delete(filePath + "temporary\\");
    }

    [WebMethod]
    public int GetArraySize()
    {
        return int.Parse(System.Configuration.ConfigurationManager.AppSettings["FileUploadArraySize"]);
    }

    protected string SetFilePath(Guid DocumentGuid)
    {
        string filePath = System.Configuration.ConfigurationManager.AppSettings["DocumentPath"];

        string[] filePaths = DocumentGuid.ToString().Split('-');

        foreach (string fp in filePaths)
        {
            filePath += fp[0] + "\\";
        }

        if (!System.IO.Directory.Exists(filePath))
        {
            System.IO.Directory.CreateDirectory(filePath);
        }

        System.IO.Directory.CreateDirectory(filePath + "temporary\\");

        return filePath;
    }

    protected string SetUrlPath(Guid DocumentGuid)
    {
        string filePath = System.Configuration.ConfigurationManager.AppSettings["DocumentUrl"];

        string[] filePaths = DocumentGuid.ToString().Split('-');

        foreach (string fp in filePaths)
        {
            filePath += fp[0] + "/";
        }

        return filePath;
    }

    protected string GetFileType(Guid DocumentGuid)
    {
        string commandText = "SELECT FileType FROM System.Document WHERE DocumentGuid = @DocumentGuid";

        System.Data.CommandType commandType = System.Data.CommandType.Text;

        connectionStringKey = "EliyahuLiteConnectionString";

        OpenSqlConnection();

        GetSqlCommand(commandText, commandType).Parameters.Add("@DocumentGuid", DocumentGuid);

        string fileType = (string)ExecuteScalar(commandText);

        ResetCommandParameters();

        CloseSqlConnection();

        return fileType;
    }

}

