#region Copyright Clause
/*
 * Blue Marble Genesis Smart Client Framework
 * 
 * Copyright  2009 Blue Marble. All rights reserved.
 * 
 * This code is protected from any unauthorized duplication or viewing under
 * international copyright laws. Unauthorized access to this code is prosecutable
 * under the full extent of the law.
 */
#endregion

using System;
using System.Web;
using System.Collections;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Data;
using System.Data.SqlClient;
using System.Configuration;


/// <summary>
/// Summary description for WebServiceBase
/// </summary>
[WebService(Namespace = "http://genesis.bluemarble.co.za/")]
[WebServiceBinding(ConformsTo = WsiProfiles.BasicProfile1_1)]
public class WebServiceBase : System.Web.Services.WebService
{
    protected string connectionStringKey;
    protected string commandText;
    protected SqlConnection connection = null;
    protected SqlCommand command;
    protected CommandType commandType = CommandType.Text;
    protected Guid sessionGuid;

    public WebServiceBase()
    {

        //Uncomment the following line if using designed components 
        //InitializeComponent(); 
    }

    public SqlConnection GetSqlConnection()
    {
        if (connection == null)
        {
            connection = new SqlConnection(ConfigurationManager.ConnectionStrings[connectionStringKey].ConnectionString);
        }
        else
        {
        }

        return connection;
    }

    public SqlCommand GetSqlCommand(string CommandText, CommandType SqlCommandType)
    {
        if (command == null)
        {
            commandText = CommandText;
            commandType = SqlCommandType;

            command = new SqlCommand(commandText, GetSqlConnection());

            command.CommandType = commandType;
        }
        else
        {
            if (command.CommandText != CommandText)
            {
                commandText = CommandText;

                command.CommandText = commandText;
            }

            if (command.CommandType != SqlCommandType)
            {
                commandType = SqlCommandType;

                command.CommandType = commandType;
            }

            if (command.Connection != connection)
            {
                command.Connection = connection;
            }
        }

        return command;
    }

    public void PrepareSqlCommandParameters()
    {
        if (command != null)
        {
            for (int i = 0; i < command.Parameters.Count; i++)
            {
                if (command.Parameters[i].Value == null)
                {
                    command.Parameters[i].Value = DBNull.Value;
                }
            }
        }
    }

    public SqlCommand GetSqlCommand(string CommandText)
    {
        return GetSqlCommand(CommandText, commandType);
    }

    public SqlDataReader ExecuteReader(string CommandText, CommandType SqlCommandType)
    {
        PrepareSqlCommandParameters();

        return GetSqlCommand(CommandText, SqlCommandType).ExecuteReader();
    }

    public SqlDataReader ExecuteReader(string CommandText)
    {
        return ExecuteReader(CommandText, commandType);
    }

    public int ExecuteNonQuery(string CommandText, CommandType SqlCommandType)
    {
        return GetSqlCommand(CommandText, SqlCommandType).ExecuteNonQuery();
    }

    public void ExecuteNonQuery(string CommandText)
    {
        ExecuteNonQuery(CommandText, commandType);
    }

    public object ExecuteScalar(string CommandText, CommandType SqlCommandType)
    {
        return GetSqlCommand(CommandText, SqlCommandType).ExecuteScalar();
    }

    public object ExecuteScalar(string CommandText)
    {
        return ExecuteScalar(CommandText, commandType);
    }

    public void ReleaseCommand()
    {
        if (command != null)
        {
            command.Dispose();

            command = null;
        }
    }

    public void OpenSqlConnection()
    {
        if (connection == null)
        {
            GetSqlConnection();
        }

        if (connection.State != ConnectionState.Open)
        {
            connection.Open();
        }
    }

    public void CloseSqlConnection()
    {
        if ((connection.State != ConnectionState.Closed) &&
            (connection != null))
        {
            connection.Close();

            connection.Dispose();
        }

        connection = null;

        ReleaseCommand();
    }

    public DataTable GenerateTableFromReader(SqlDataReader Reader, string TableName)
    {
        DataTable resultValue = new DataTable(TableName);

        for (int i = 0; i < Reader.FieldCount; i++)
        {
            DataColumn column = new DataColumn(Reader.GetName(i), Reader.GetFieldType(i));

            resultValue.Columns.Add(column);
        }

        while (Reader.Read())
        {
            DataRow row = resultValue.NewRow();

            for (int i = 0; i < Reader.FieldCount; i++)
            {
                DataColumn column = resultValue.Columns[Reader.GetName(i)];

                row[column] = Reader.GetValue(i);
            }

            resultValue.Rows.Add(row);
        }

        return resultValue;
    }

    public DataTable SelectManyLookup(string TableName, string ValueColumnName, string DisplayColumnName)
    {
        DataTable resultValue = null;

        string selectManyLookupCommandText = "SelectManyLookup";

        OpenSqlConnection();

        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.Add(new SqlParameter("@TableName", TableName));
        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.Add(new SqlParameter("@ValueColumnName", ValueColumnName));
        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.Add(new SqlParameter("@DisplayColumnName", DisplayColumnName));

        resultValue = GenerateTableFromReader(ExecuteReader(selectManyLookupCommandText, CommandType.StoredProcedure), TableName);

        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.RemoveAt(2);
        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.RemoveAt(1);
        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.RemoveAt(0);

        CloseSqlConnection();

        return resultValue;
    }

    public DataTable SelectManyLookup(string TableName, string ValueColumnName, string DisplayColumnName, string WhereClause)
    {
        DataTable resultValue = null;

        string selectManyLookupCommandText = "SelectManyLookup";

        OpenSqlConnection();

        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.Add(new SqlParameter("@TableName", TableName));
        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.Add(new SqlParameter("@ValueColumnName", ValueColumnName));
        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.Add(new SqlParameter("@DisplayColumnName", DisplayColumnName));

        resultValue = GenerateTableFromReader(ExecuteReader(selectManyLookupCommandText, CommandType.StoredProcedure), TableName);

        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.RemoveAt(2);
        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.RemoveAt(1);
        GetSqlCommand(selectManyLookupCommandText, CommandType.StoredProcedure).Parameters.RemoveAt(0);

        CloseSqlConnection();

        return resultValue;
    }

    public Guid AuthorizeSession(Guid SessionGuid)
    {
        sessionGuid = SessionGuid;

        Guid resultValue = Guid.Empty;

        string authorizeSessionCommandText = "AuthorizeSession";

        this.ConnectionStringKey = "GenesisConnectionString";

        OpenSqlConnection();

        GetSqlCommand(authorizeSessionCommandText, CommandType.StoredProcedure).Parameters.Add(new SqlParameter("@SessionGuid", SessionGuid));

        resultValue = (Guid)ExecuteScalar(authorizeSessionCommandText, CommandType.StoredProcedure);

        GetSqlCommand(authorizeSessionCommandText, CommandType.StoredProcedure).Parameters.RemoveAt(0);

        CloseSqlConnection();

        return resultValue;
    }

    public Guid AuthorizeTransaction(Guid TransactionGuid)
    {
        return Guid.NewGuid();
    }

    public string ConnectionStringKey
    {
        get
        {
            return connectionStringKey;
        }

        set
        {
            connectionStringKey = value;
        }
    }

    public void ResetCommandParameters()
    {
        while (GetSqlCommand(commandText, commandType).Parameters.Count > 0)
        {
            GetSqlCommand(commandText, commandType).Parameters.RemoveAt(0);
        }
    }
}

